;;;
;;; <util.ss> ---- Utilerias para el MonitoAd
;;;
;;; Copyright (C) 2001 by Karla Ramrez. 
;;;
;;; This file is part of MonitoAd.

;;; MonitoAd is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Lesser General Public
;;; License as published by the Free Software Foundation; either
;;; version 2.1 of the License, or (at your option) any later version.

;;; MonitoAd is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Lesser General Public License for more details.

;;; You should have received a copy of the GNU Lesser General Public
;;; License along with MonitoAd; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

;;; Author: Karla Ramrez <karla@abulafia.fciencias.unam.mx>
;;
;;
;; Commentary:

;; string-index: cadena caracter -> natural
;; regresa el ndice ms a la izquierda en la cadena s, que tiene al
;; caracter c.
(define string-index
  (lambda (s c)
    (let ((n (string-length s)))
      (let loop ((i 0))
	(cond ((>= i n) #f)
	      ((char=? (string-ref s i) c) i)
	      (else (loop (+ i 1))))))))

;; string-tokenizer: caracter cadena -> lista
;; string-tokenizer rompe la cadena s en las posiciones donde
;; encuentra un caracter c y regresa una lista con esas subcadenas.
(define string-tokenizer
  (lambda (c s)
    (let loop ((s s))
      (if (string=? s "") '()
	  (let ((i (string-index s c)))
	    (if i (cons (substring s 0 i)
			(loop (substring s (+ i 1)
					 (string-length s))))
		(list s)))))))

;; log-time : () -> time stamp
;; regresa la fecha actual tipo Log de Apache (es slo para guardar un
;; log de lo que hace el servidor).
(define log-time
  (lambda ()
    (let* ((date (seconds->date (current-seconds)))
	   (day (list-ref '("Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat")
			  (date-week-day date)))
	   (month (list-ref '("Jan" "Feb" "Mar" "Apr" "May"
			      "Jun" "Jul" "Aug" "Sep" "Oct"
			      "Nov" "Dec")
			    (sub1 (date-month date)))))
      (format "[~a ~a ~a ~a:~a:~a ~a]" day month (date-day date)
	      (date-hour date) (date-minute date) (date-second date)
	      (date-year date)))))

;; log-this : cadena cadena -> void
;; escribe el mensaje msg, con la fecha actual en el archivo de logs.
(define log-this
  (lambda (level msg)
    (fprintf (current-error-port)
	     "~a [~a] ~a~n" (log-time) level msg)))

;; create-dummy-password : void -> String
;; Regresa una cadena de longitud 8 con caracteres ASCII
;; enre el caracter ASCII 48 (0) y el 126 (~).  Sirve como
;; generadora de passwords aleatorios (temporales) para los
;; administradores.
(define create-dummy-password
  (lambda ()
    (list->string
     (let loop ((count 8))
       (if (zero? count) ()
	   (cons (integer->char (+ (random 78) 48))
		 (loop (sub1 count))))))))		   
    
;; is-password-cool: String -> Boolean
;; Regresa #t si PASS es lo suficientemente seguro o
;; #f si es lo suficientemente chafa como para no
;; aceptarlo.
(define is-password-cool
  (lambda (pass)
    #t))

(define ->string
  (lambda (item)
    (cond ((schemeql-timestamp? item)
	   (format "~a/~a/~a ~a:~a"
		   (schemeql-timestamp-year item)
		   (schemeql-timestamp-month item)
		   (schemeql-timestamp-day item)
		   (schemeql-timestamp-hour item)
		   (schemeql-timestamp-minute item)))
	  (else (format "~a" item)))))

(define timestamp-time
  (lambda (item)
    (format "~a:~a" (schemeql-timestamp-hour item)
	    (schemeql-timestamp-minute item))))


(define flatten
  (lambda (lst)
    (cond ((null? lst) '())
	  (else (append (car lst) (flatten (cdr lst)))))))

;;; util.ss ends here