;; Procedimientos para manejar a los administradores.

;; Todo depende de la funcionalidad del servidor de Web
;; de PLT Scheme, ya que una vez que el administrador hace
;; "login", las rutinas en este archivo se encargan de
;; presentarle todas sus opciones y atender sus peticiones:

;; sismo-login: (list-of bindings) -> (values Boolean (U list string))
;; 
;; Recibe una lista de bindings (donde en particular, slo nos
;; interesan los bindings de login y password).  Se conecta con el
;; servidor del SisMo y si el servidor regresa una cadena, entonces es
;; una cadena que indica un "error" y regresa el par de valores (#f
;; "error") para indicar esta situacin, en caso contrario regresa (#t
;; (id "nombre")).
(define sismo-login
  (lambda (b)
    (let* ((login (extract-binding 'login b))
	   (passwd (extract-binding 'passwd b))
	   (ans (sismo-client (format "login:~a:~a" login passwd))))
      (values (not (string? ans)) ans))))

;; generate-continuation-menu: Url-string -> X-expression (menu)
;;
;; Recibe un continuation (una liga a una continuacin de ejecucin de
;; un CGI) y lo utiliza para generar los distintos valores posibles de
;; acciones a seguir.  Este men se puede ver y ejecutar slo durante
;; determinado tiempo, tal y como es especificado por
;; `adjust-timeout', por omisin 30 minutos (en segundos)
(define generate-continuation-menu
  (lambda (link)
    `((table ((cellspacing "0") (cellpadding "1"))
	     (tbody ()
		    (tr ((valign "top") (bgcolor "#6495ED"))
			(td ((align "center"))
			    (span ((class "label"))
				  "Men principal")))
		    (tr ()
			(td ((bgcolor "#6495ED"))
			    (table ((width "100%") (cellspacing "0") (cellpadding "5"))
				   (tbody ()
					  (tr ()
					      (td ((bgcolor "#ffffff") (align "center"))
						  (hr ())
						  (span ((class "head")) "Hosts:") (br ())
						  (a ((href ,(format "~a?action=add&what=host" link))) "agregar") (br ())
						  (a ((href ,(format "~a?action=list&what=host" link))) "listar") (br ())
						  (hr ())
						  (span ((class "head")) "Admin:") (br ())
						  (a ((href ,(format "~a?action=add&what=admin" link))) "agregar") (br ())
						  (hr ())
						  (span ((class "head")) "Opciones:") (br ())
						  (a ((href ,(format "~a?action=logout" link))) "salir") (br ())
						  (a ((href ,(format "~a?action=changepass&step=form" link))) "cambiar password") (br ())
						  (hr ())
						  (span ((class "head")) "Ayuda:") (br ())
						  (a ((href ,(format "~a?action=about" link))) "acerca de") (br ())
						  (hr ())
						   
						  )))))))))))

(define sismo-main-page
  (lambda (data)
    (let ((uid (car data))
	  (name (cadr data)))
      ;; Media hora para que el administrador haga lo que tenga que hacer.
      (adjust-timeout 1800)
      (generate-logged-in-page
       uid
       `((h1 "Bienvenido(a) a SisMo")
	 (p "Estimado(a): "
	    (strong ,name) ". Por favor selecciona alguna opcin del men a la izquierda."))))))


;; sismo-add-host-form: Continuation -> Xexpr
;; Recibe un continuation (para continuar la ejecucin) y genera el
;; cuerpo de una pgina con una forma para agregar un host al sistema.
(define sismo-add-host-form
  (lambda (link)
    `((h1 "Aade un host para monitorear")
      (form ((action ,link) (method "post"))
	    (input ((type "hidden") (name "action") (value "addhost")))
	    (table ((border "1") (align "center"))
		   (tbody ()
			  (tr ()
			      (td ((align "right")) "Hostname:")
			      (td ((align "left")) (input ((name "hostname") (size "50") (maxlength "128")))))
			  (tr ()
			      (td ((align "right")) "IP:")
			      (td ((align "left")) (input ((name "ip") (size "15") (maxlength "15")))))
			  (tr ()
			      (td ((colspan "2") (align "center") (height "50"))
				  (input ((type "submit") (value " agregar ")))))))))))

;; sismo-add-admin-form: Continuation -> Xexpr
;; Recibe un continuation (para continuar la ejecucin) y genera el
;; cuerpo de una pgina con una forma para agregar un host al sistema.
(define sismo-add-admin-form
  (lambda (link)
    `((h1 "Aade un administrador")
      (form ((action ,link) (method "post"))
	    (input ((type "hidden") (name "action") (value "addadmin")))
	    (table ((border "1") (align "center"))
		   (tbody ()
			  (tr ()
			      (td ((align "right")) "login:")
			      (td ((align "left")) (input ((name "login") (size "15") (maxlength "16")))))
			  (tr ()
			      (td ((align "right")) "nombre:")
			      (td ((align "left")) (input ((name "name") (size "50") (maxlength "75")))))
			  (tr ()
			      (td ((colspan "2") (align "center") (height "50"))
				  (input ((type "submit") (value " agregar ")))))))))))

(define sismo-change-form
  (lambda (link)
    `((h1 "Cambia tu password")
      (p "En la siguiente forma escribe tu password actual y dos veces el nuevo password, gracias.")
      (form ((action ,link) (method "post"))
	    (input ((type "hidden") (name "action") (value "changepass")))
	    (input ((type "hidden") (name "step") (value "2")))
	    (table ((border "1") (align "center"))
		   (tbody ()
			  (tr ()
			      (td ((align "right")) "Password Actual:")
			      (td ((align "left")) (input ((type "password") (name "old") (size "15") (maxlength "16")))))
			  (tr ()
			      (td ((align "right")) "Password Nuevo:")
			      (td ((align "left")) (input ((type "password") (name "new") (size "15") (maxlength "16")))))
			  (tr ()
			      (td ((align "right")) "Confirma Password Nuevo:")
			      (td ((align "left")) (input ((type "password") (name "confirm") (size "15") (maxlength "16")))))
			  (tr ()
			      (td ((colspan "2") (align "center") (height "50"))
				  (input ((type "submit") (value " cambiar ")))))))))))

;; sismo-get-hosts: Nat Continuation-string -> Xexpr
;; Regresa una tabla con todos los hosts que administra el
;; administrador con un valor UID.
(define sismo-get-hosts
  (lambda (uid link)
    (let ((hosts (sismo-client (format "getmyhosts:~a" uid))))
      `((p "En la siguiente lista encontrars las direcciones IP y los nombres de los equipos que ests monitoreando.  Si la lista es vaca, o deseas monitorear un equipo linux ms, usa la opcin de " (strong "aadir") " (debajo de " (em "hosts") "), en el men de la izquierda.  Gracias.")
	(table ((align "center") (border "1") (cellpadding "5"))
	       (tbody ()
		      (tr ((bgcolor "#6495ED"))
			  (td ((align "center")) (span ((class "label")) "IP"))
			  (td ((align "center")) (span ((class "label")) "Hostname")))
		      ,@(map (lambda (h)
			       `(tr ()
				    (td ((align "left")) (a ((href ,(format "~a?action=show&host=~a"
									     link (list-ref h 0))))
							     ,(list-ref h 1)))
				    (td ((align "left")) (a ((href ,(format "~a?action=show&host=~a"
									    link (list-ref h 0))))
							    ,(list-ref h 2)))))
			     hosts)))
	(p "Para ver los detalles de un " (em "host") ", has click en su direccin IP o en su nombre.")))))

(define areas-alist
  '(("memory" "Memoria" "Uso de la memoria principal y memoria virtual del sistema.")
    ("processor" "Procesador" "Carga del procesador.")
    ("process" "Procesos" "Procesos de usuarios y del sistema.")
    ("fs" "Sistema de Archivos" "Uso de los sistemas de archivos montados en el host.")
    ("users" "Usuarios" "Usuarios en actividad en el host.")
    ("network" "Red" "Transmisin de paquetes en cada interfaz del host.")
    ("nfs" "NFS" "Network File System: sistemas montados y exportados.")
    ("printer" "Impresoras" "Estado de las impresoras instaladas y trabajos formados.")))

(define sismo-show-host
  (lambda (link hostid)
    (let* ((h (sismo-client (format "searchhost:~a:id" hostid)))
	   (hostname (if (null? h)
			 "error en hostname"
			 (list-ref (car h) 2))))
      `((h1 ,hostname)
	(p "Escoge una opcin de la siguiente tabla.")
	(table ((align "center") (border "1") (cellpadding "5") (width "95%"))
	       (tbody ()
		      (tr ((bgcolor "#6495ED"))
			  (td ((align "center")) (span ((class "label")) "rea"))
			  (td ((align "center")) (span ((class "label")) "Descripcin [y opciones]")))
		      ,@(map (lambda (area)
			       `(tr ()
				    (td ((align "left"))
					(a ((href ,(format "~a?action=pickdate&host=~a&area=~a"
							   link hostid (car area))))
					   ,(cadr area)))
				    (td ((align "left")) ,(caddr area))))
			     areas-alist)))))))


(define pick-date
  (lambda (link hostid area)
    (let* ((h (sismo-client (format "searchhost:~a:id" hostid)))
	   (hostname (if (null? h)
			 "error en hostname"
			 (list-ref (car h) 2)))
	   (today (seconds->date (current-seconds)))
	   (year (date-year today))
	   (m (date-month today))
	   (day (date-day today))
	   )
      `((h1 ,hostname)
	(p "Escoge una fecha para que te muestre la informacin almacenada por el equipo: "
	   (strong ,hostname)
	   " en dicho da.")
      (form ((action ,link) (method "post"))
	    (input ((type "hidden") (name "action") (value "graph")))
	    (input ((type "hidden") (name "host") (value ,hostid)))
	    (input ((type "hidden") (name "area") (value ,area)))
	    (table ((align "center") (border "1") (cellpadding "5") (width "95%"))
		   (tbody ()
		    (tr ((bgcolor "#6495ED"))
			(td ((align "center")) (span ((class "label")) "Ao"))
			(td ((align "center")) (span ((class "label")) "Mes"))
			(td ((align "center")) (span ((class "label")) "Da")))
		    (tr ()
			(td ((align "center"))
			    (select ((name "year"))
				    ,@(let loop ((i 0))
					(if (= i 5)
					    '()
					    (cons
					     `(option () ,(number->string (- year i)))
					     (loop (add1 i)))))))
			(td ((align "center"))
			    (select ((name "month"))
				    ,@(let loop ((months month-list) (i 1))
					(cond ((null? months) '())
					      (else
					       (cons
						(if (= i m)
						    `(option ((selected "") (value ,(number->string i))) ,(car months))
						    `(option ((value ,(number->string i))) ,(car months)))
						(loop (cdr months) (add1 i))))))))
			(td ((align "center"))
			    (select ((name "day"))
				    ,@(let loop ((i 1))
					(if (= i 32)
					    '()
					    (cons
					     (if (= i day)
						 `(option ((selected "")) ,(number->string i))
						 `(option () ,(number->string i)))
						  (loop (add1 i))))))))
		    (tr ()
			(td ((colspan "3") (align "center") (height "50"))
			    (input ((type "submit") (value " ir "))))))))))))

;; store-host : Natural String String -> Xexpr
;; Recibe el ID de un administrador, un hostname y la direccin IP de
;; dicho host y los agrega (si es posible) a las tablas del sismo para
;; ser monitoreados.  Regresa el cuerpo de una pgina con el resultado
;; de la operacin.
(define store-host
  (lambda (uid hostname ip)
    (let ((h (string-tokenizer #\. hostname))
	  (i (string-tokenizer #\. ip)))
      (if (and
	   ;; Si el hostname est bien formado (i.e. es vlido)
	   (andmap valid-name h)
	   ;; y si la direccin IP es vlida:
	   (andmap valid-ip-block i))
	  (begin
	    ;; Entonces guardamos los datos y los asociamos con el
	    ;; administrador que pidi guardar este host.
	    (sismo-client (format "addhost:~a:~a:~a" uid ip hostname))
	    ;; avisamos que todo sali bien:
	    `((h1 "Operacin exitosa")
	      (p "El host " (strong ,hostname) ", fue aadido exitosamente.")))
	  ;; Marcamos un error, porque el hostname o el IP no son
	  ;; vlidos:
	  `((h1 "Error")
	    (p "Hay algo mal con el nombre del host o con la direccin IP, por favor regresa y revisa tus datos, gracias."))))))

;; store-admin : String String -> Xexpr
;; Agrega un nuevo administrador al sismo.  Ntese que cualquier
;; administrador puede agregar otros administradores.  Eventualmente
;; sera buena idea tener "niveles" de administracin.
(define store-admin
  (lambda (login name)
    (let ((pass (sismo-client (format "addadmin:~a:~a" login name))))
      (if (regexp-match "^error" pass)
	  `((h1 "Error")
	    (p "Ese login ya existe.  Por favor, regresa y cambia el login (si an quieres agregar al usuario)."))
	  `((h1 "Operacin exitosa")
	    (p "El usuario, " (strong ,login) ", ha sido aadido exitosamente.  Por favor toma nota, el password del nuevo usuario es:")
	    (h1 ,pass)
	    (br ())
	    (p "Ms adelante el usuario mismo podr cambiar su password."))))))

(define sismo-change-password
  (lambda (uid old new)
    `((h1 "Cambio de password")
      (p "Tus datos fueron enviados al servidor del Sismo y esta es la respuesta: ")
      (br ())
      (h1 ,(sismo-client (format "changepasswd:~a:~a:~a" uid old new))))))

(define list-of-colors '("#000000" ;; negro
			 "#F08080" ;; rojo
			 "#8DBC8F" ;; verde
			 "#0000FF" ;; azul
			 "#FFD700" ;; dorado
			 "#800080" ;; morado
			 "#800000" ;; marrn
			 "#87CEED" ;; azul cielo
			 "#FF69B4" ;; hot pink
			 "#FFA500" ;; anaranjado 
			 ))

;; Esta variable tiene un mapeo de reas del sismo a nmeros.  Estos
;; nmeros son usados para cambiar los ttulos de las grficas en el
;; programa en C.
(define area-number-alist
  '((fs . "0") ;; Sistema de Archivos
    (processor . "1")  ;; Procesador
    (process . "2") ;; Procesos
    (memory . "3") ;; Memoria
    (users . "4") ;; usuarios
    (network . "5") ;; red
    (nfs . "6") ;; nfs
    (printer . "7")))

(define sismo-show-graph
  (lambda (link hostid area from to)
    (let* ((to-graph (sismo-client (format "get:~a:~a:~a:~a" hostid area from to)))
	   (h (sismo-client (format "searchhost:~a:id" hostid)))
	   (hostname (if (null? h)
			 "error en hostname"
			 (list-ref (car h) 2)))
	   (filename (format "~a-~a" area (current-seconds)))
	   (input (string-append tmp-directory filename ".data"))
	   (output (string-append tmp-directory filename ".gif"))
	   )
      (if (zero? (car to-graph)) ;; Nmero de puntos
	  `((h1 ,hostname)
	    (p "No hay nada que mostrar en esta categora."))
	  (let ((points (list-ref to-graph 0))
		(lines (list-ref to-graph 1))
		(labels (list-ref to-graph 2))
		(all-lines (list-ref to-graph 3))
		(x-labels (list-ref to-graph 4)))
	    ;; Primero generamos el archivo que el programa que
	    ;; grafica va a leer:
	    (call-with-output-file input
	      (lambda (p)
		(write points p)
		(display #\tab p)
		(write lines p)
		(newline p)
		(let loop ((xs x-labels))
		  (unless (null? xs)
			  (display (car xs) p)
			  (display #\tab p)
			  (loop (cdr xs))))
		(newline p)
		;; Ahora los datos:
		(map (lambda (l)
		       (map (lambda (n)
			      (write n p)
			      (display #\tab p))
			    l)
		       (newline p))
		     all-lines))
	      'replace)
	    ;; Ahora, llamamos al programa que grafica:
	    (system (string-append graph-maker " " (cdr (assq
							 (if (string? area)
							     (string->symbol area) area)
							 area-number-alist)) " " input " " output))
	    ;; Y mostramos los resultados:
	    `((h1 ,hostname)
	      (table ((width "90%") (cellpadding "5") (cellspacing "0"))
		     (tbody ()
			    (tr ()
				(td ((colspan "2") (bgcolor "#6495ED") (align "center") (valign "middle"))
				    (span ((class "label"))
					  ,(cadr (assoc area areas-alist)))))
			    (tr ()
				(td ((align "center") (valign "middle"))
				    (img ((src ,(string-append "/tmp/" filename ".gif")) (border "0"))))
				(td ((align "center") (valign "middle"))
				    (table ((width "98%") (align "center") (valign "middle"))
					   (tbody ()
						  ,@(let ((index 0)
							  (len (length list-of-colors)))
						      (map (lambda (l)
							     `(tr ()
								  (td ((width "30")
								       (bgcolor ,(let ((c (list-ref list-of-colors (modulo index len))))
										   (set! index (add1 index))
										   c))))
								  (td () (a ((href ,(format "~a?action=gettable&host=~a&area=~a&label=~a&from=~a&to=~a"
											    link hostid area l from to)))
									    ,l))))
							   labels))))))))))))))

(define sismo-show-table
  (lambda (link hostid area label from to)
    (let* ((data (sismo-client (format "gettable:~a:~a:~a:~a:~a"
				       hostid area label from to)))
	   (h (sismo-client (format "searchhost:~a:id" hostid)))
	   (hostname (if (null? h)
			 "error en hostname"
			 (list-ref (car h) 2)))
	   )
      (if (null? data)
	  `((h1 "No hay datos disponibles")
	    (p "Error: los datos son incorrectos, por favor revisa la informacin."))
	  `((h1 ,hostname)
	    (table ((width "90%") (cellpadding "5") (cellspacing "0"))
		       (tbody ()
			      (tr ()
				  (td ((bgcolor "#6495ED") (align "center") (valign "middle"))
				      (span ((class "label"))
					    ,(format "~a : ~a"
						     (cadr (assoc area areas-alist))
						     label))))
			      (tr ()
				  (td ((align "center") (valign "middle"))
				      (table ((width "98%") (align "center") (valign "middle") (border "2")
					      (cellpadding "3") (cellspacing "3"))
					     (tbody ()
						    ;; Primero ponemos las etiquetas que nos enva el sismo:
						    (tr ()
							,@(map (lambda (l)
								 `(th ((align "center")) (strong ,l)))
							       (car data)))
						    ;; Y ahora ponemos la tabla con la informacin:
						    ,@(map (lambda (ln)
							     `(tr ()
								  ,@(map (lambda (l)
									   `(td ((align "center")) ,l))
									 ln)))
							   (cdr data)))))))))))))

;; Para generar dos dias como:
;; "2002-01-03 00.00"
;; "2002-01-03 24.00"
(define make-date-from
  (lambda (year month day)
    (let* ((y (string-append
	      (if year
		  year
		  (number->string (date-year (seconds->date (current-seconds)))))
	      "-"
	      (if month
		  month
		  (number->string (date-month (seconds->date (current-seconds)))))
	      "-"
	      (if day
		  day
		  (number->string (date-day (seconds->date (current-seconds))))))))
      (values (string-append y " 00.00")
	      (string-append y " 23.59")))))

;; generate-logged-in-page: Nat Xexpression -> continuation Esta
;; funcin recibe un nmero natural, el UID (user ID) de un usuario
;; que exitosamente logr entrar al Sismo.  A partir de este momento
;; le atenderemos slo con continuations como medida de seguridad, ya
;; que estos se generan dinmicamente.  Para mayores detalles consulte
;; el captulo 10 de la tesis: "SisMo: Un sistema de monitoreo para
;; estaciones Linux".
(define generate-logged-in-page
  (lambda (uid body)
    (let ( ;; Link es una pequea trampa.  Ser una liga que siempre
	  ;; apunte al continuation actual (i.e. uno menos que el
	  ;; continuation al que se dirigen la mayora de las ligas en
	  ;; el men, pero no es til para poder controlar a todas las
	  ;; dems ligas (dinmicas), por ejemplo, las que generan
	  ;; llamadas a "sismo-show-host", etc.
	  (link "/cgi-bin/sismo.ss"))
      (let-values
       ([(method url headers bindings)
	 (send/suspend
	  (lambda (k-url)
	    (set! link k-url)
	    ;; Media hora para que el administrador haga lo que tenga que hacer.
	    (adjust-timeout 1800)
	    `(html
	      (head
	       (meta ((name "author") (content "Karla Ramrez (SisMo, powered by PLT Scheme)")))
	       (meta ((name "keywords") (content "SisMo, sistema de monitoreo")))
	       (link ((href "/style.css") (type "text/css") (rel "stylesheet")))
	       (title "SisMo: Sistema de monitoreo para estaciones de trabajo Linux"))
	      (body ((bgcolor "#ffffff"))
		    ,@title
		    (table ((width "100%") (border "0") (cellpadding "5"))
			   (tbody ()
				  (tr ()
				      (td ((valign "top") (width "200"))
					  ,@(generate-continuation-menu k-url))
				      (td ((valign "top") (width "100%"))
					  ,@body))
				  ,@footer))))))])
       (let ((action (extract-binding 'action bindings))
	     (real-body `((p (strong "Error:") " operacin no soportada."))))
	 (when action
	       (cond ((string=? action "about")
		      (set! real-body (read-page "about")))
		     ((string=? action "help")
		      (set! real-body (read-page "help")))
		     ;; Lista de hosts administrados por el usuario con UID:
		     ((string=? action "list")
		      (set! real-body (sismo-get-hosts uid link)))
		     ;; Agregar algo (host o administrador);
		     ((string=? action "add")
		      (let ((what (extract-binding 'what bindings)))
			(cond (;; Agregar host:
			       (string=? what "host")
			       (set! real-body (sismo-add-host-form link)))
			      (;; Agregar administrador:
			       else
			       (set! real-body (sismo-add-admin-form link))))))
		     ;; Agregando los datos de un nuevo host:
		     ((string=? action "addhost")
		      (let ((hostname (extract-binding 'hostname bindings))
			    (ip (extract-binding 'ip bindings)))
			(set! real-body
			      (if (and hostname ip)
				  (store-host uid hostname ip)
				  `((h1 "Error")
				    (p "Debes pasar tanto el hostname
como la direccin IP del host a monitorear.  Por favor regresa e
intntalo de nuevo."))))))
		     ;; Agregando los datos de un nuevo administrador:
		     ((string=? action "addadmin")
		      (let ((login (extract-binding 'login bindings))
			    (name (extract-binding 'name bindings)))
			(set! real-body
			      (if (and login name)
				  (store-admin login name)
				  `((h1 "Error")
				    (p "Debes pasar tanto el login,
como el nombre del administrador que quieres agregar.  Por favor
regresa e intntalo de nuevo."))))))
		     ;; Cambiar password
		     ((string=? action "changepass")
		      (let ((step (extract-binding 'step bindings)))
			(when
			 step
			 (set! real-body
			       (cond ((string=?  step "form")
				      (sismo-change-form link))
				     (else
				      (let ((old (extract-binding 'old bindings))
					    (new (extract-binding 'new bindings))
					    (confirm (extract-binding 'confirm bindings)))
					(if (and old new confirm)
					    (if (string=? new confirm)
						(sismo-change-password uid old new)
						`((h1 "Error")
						  (p "El nuevo
password NO concuerda.  Por favor, regresa y teclea con cuidado (ambas
veces) el nuevo password, gracias.")))
					    `((h1 "Error")
					      (p "Para poder cambiar
tu password, debes llenar todos los campos: tu password actual, y dos
veces el password nuevo.  Por favor, regresa y vuelve a
intentarlo."))))))))))
		     ;; Para mostrar un host:
		     ((string=? action "show")
		      (let ((host (extract-binding 'host bindings)))
			(when (and host
				   (string? host))
			      (set! real-body
				    (sismo-show-host link (string->number host))))))
		     ;; Para seleccionar el da a desplegar la informacin:
		     ((string=? action "pickdate")
		      (let ((host (extract-binding 'host bindings))
			    (area (extract-binding 'area bindings)))
			(when (and host area)
			      (set! real-body (pick-date link host area)))))
		     ;; Para generar las grficas:
		     ((string=? action "graph")
		      (let ((host (extract-binding 'host bindings))
			    (area (extract-binding 'area bindings))
			    (year (extract-binding 'year bindings))
			    (month (extract-binding 'month bindings))
			    (day (extract-binding 'day bindings))
			    )
			(when (and host area)
			      (let-values
			       ([(from to) (make-date-from year month day)])
			       (set! real-body
				     (sismo-show-graph link host area from to))))))
		     ;; Para mostrar las tablas de datos:
		     ((string=? action "gettable")
		      (let ((host (extract-binding 'host bindings))
			    (area (extract-binding 'area bindings))
			    (label (extract-binding 'label bindings))
			    (from (extract-binding 'from bindings))
			    (to (extract-binding 'to bindings)))
			(when (and host area label)
			      (set! real-body
				    (sismo-show-table link host area label from to))))))
	       ;; Si action es "logout", entonces ya no enviamos un
	       ;; nuevo continuation, simplemente regresamos a la
	       ;; funcin que nos llam (otro continuation o
	       ;; `sismo-login', la primera funcin que gener el
	       ;; continuation.
	       (unless (string=? action "logout")
		       (generate-logged-in-page uid real-body))))))))
