;; Utilidades del SisMo

;; Strings:

;; trim: String -> String
;; Slo elimina los blancos al inicio y fin de la cadena.
(define trim
  (lambda (str)
    (trim-right (trim-left str))))

(define trim-left
  (lambda (str)
    (let* ((r (regexp "^[ 	]*(.*)"))
	   (ans (regexp-match r str)))
      (and ans (regexp-replace r str "\\1")))))

(define trim-right
  (lambda (str)
    (let* ((r (regexp "[ 	]*$"))
	   (pos (regexp-match-positions r str)))
      (substring str 0 (caar pos)))))

;; extract-binding: Symbol (list-of bindings) -> (listof Strings)
;; Si encuentra un binding para NAME en la lista de bindings B,
;; regresa la(s) cadena(s) asociada(s).
(define extract-binding
  (lambda (name b)
    (call/cc
     (lambda (exit)
       (let loop ((b b))
	 (cond ((null? b) (exit #f))
	       ((eq? name (caar b)) (trim (cdar b)));; found!
	       (else
		(loop (cdr b)))))))))

;; sismo-client : String  param [param ...] -> String
(define sismo-client
  (lambda (stuff)
    (let-values
     ([(ip op)
       (tcp-connect (sismo-server-name) (sismo-port))])
     (dynamic-wind void
		   (lambda ()
		     ;; Enviando informacin al demonio:
		     (display stuff op)
		     (newline op)
		     ;; Leyendo la respuesta:
		     (read ip))
		   (lambda ()
		     (close-input-port ip)
		     (close-output-port op))))))

;; Ttulo del SisMo
(define title
  `((table
     ((width "100%") (cellspacing "0") (cellpadding "1"))
     (tbody ()
	    (tr ()
		(td ((valign "middle") (align "center"))
		    (span ((class "title"))
			  "SisMo: Sistema de monitoreo para estaciones de trabajo Linux"))
		(td ((valign "middle") (width "110") (height "100"))
		    (img ((src "/img/caban-small.gif")
			  (alt "Logo")
			  (border "0")))))))))

;; Pie de las pginas
(define footer
  `((tr ()
	(td ((colspan "2") (align "center")
	     (height "40") (valign "bottom"))
	    (div ((class "copyright"))
		 copy "2002 " (a ((href "mailto:karla@abulafia.fciencias.unam.mx"))
				 "Karla Ramrez Pulido"))))))

;; Menus:
(define menu-alist
  '(("default" . ((hr ())
		  (strong "Hosts:") (br ())
		  (a ((href "/cgi-bin/sismo.ss?action=list&what=host")) "listar") (br ())
		  (a ((href "/cgi-bin/sismo.ss?action=add&what=host")) "agregar") (br ())
		  (hr ())
		  (strong "Admin:") (br ())
		  (a ((href "/cgi-bin/sismo.ss?action=add&what=admin")) "agregar") (br ())
		  (hr ())
		  (strong "Opciones:") (br ())
		  (a ((href "/cgi-bin/sismo.ss?action=logout")) "salir") (br ())
		  (hr ())
		  (strong "Ayuda:") (br ())
		  (a ((href "/cgi-bin/sismo.ss?action=help")) "acerca de...") (br ())
		  (hr ())
		  ))
    ("error" . ((hr ())
		(a ((href "/index.html")) "introduccin") (br ()) (hr ())
		(a ((href "/faq.html")) "FAQ") (br ()) (hr ())
		(a ((href "/start.html")) "entrar") (br ())
		))
    ))

(define read-menu
  (lambda (menu header)
    `((table ((cellspacing "0") (cellpadding "2"))
	     (tbody ()
		    (tr ((valign "top") (bgcolor "#6495ED"))
			(td ((align "center") (valign "middle") (height "40"))
			    (span ((class "label")) ,header)))
		    (tr ()
			(td ((bgcolor "#6495ED"))
			    (table ((width "100%") (cellspacing "0") (cellpadding "4"))
				   (tbody ()
					  (tr ()
					      (td ((bgcolor "#ffffff") (align "center"))
						  ,@(cdr (assoc menu menu-alist))
						  )))))))))))

(define support-dir "/home/karla/tesis/server/www/cgi-bin/static/")
(define read-real-file
  (lambda (txt-file)
    (call-with-input-file (string-append support-dir txt-file)
      (lambda (port)
	(let loop ((thing (read port)))
	  (cond ((eof-object? thing) ())
		(else
		 (cons thing (loop (read port))))))))))

(define read-page
  (lambda (file)
    (list `(table ((width "95%") (cellspacing "0") (cellpadding "1") (align "center"))
		  (tbody ()
			 (tr ()
			     (td ((bgcolor "#6495ED"))
				 (table ((width "100%") (cellspacing "0") (cellpadding "10"))
					(tbody ()
					       (tr ()
						   (td ((bgcolor "#ffffff"))
						       ,@(read-real-file file))))))))))))

;; string-index returns the leftmost index in string s
;; that has character c
(define string-index
  (lambda (s c)
    (let ((n (string-length s)))
      (let loop ((i 0))
	(cond ((>= i n) #f)
	      ((char=? (string-ref s i) c) i)
	      (else (loop (+ i 1))))))))

;; string-tokenizer breaks string s into substrings separated by character c
(define string-tokenizer
  (lambda (c s)
    (let loop ((s s))
      (if (string=? s "") '()
	  (let ((i (string-index s c)))
	    (if i (cons (substring s 0 i)
			(loop (substring s (+ i 1)
					 (string-length s))))
		(list s)))))))

;; valid-name : String -> (U String #f)
;; Esta funcin es usada para determinar si el nombre de un host o un
;; (sub)dominio es vlido o no.
(define valid-name
  (lambda (name)
    (let* ((r (regexp "^[a-zA-Z]+[0-9]*$"))
	   (match (regexp-match r name)))
      (if (and match
	       (string=? name (car match)))
	  name
	  #f))))

;; valid-ip-block : String -> (U Natural #f)
;; Esta funcin es usada para determinar si un bloque (una secuencia
;; de nmeros en una direccin IP) es vlida o no.
(define valid-ip-block
  (lambda (num)
    (let ((n (string->number num)))
      (if (and n (> n 0) (< n 256)) n #f))))


(define month-list
  '("Enero" "Febrero" "Marzo" "Abril" "Mayo" "Junio" "Julio"
    "Agosto" "Septiembre" "Octubre" "Noviembre" "Diciembre"))
