/*
 * <graph.c> ---- Funciones de soporte para generar los GIFs del SisMo
 *
 * Copyright (C) 2001 by Free Software Foundation, Inc.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with XEmacs; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * @author Karla Ramrez Pulido <karla@abulafia.fciencias.unam.mx>
 * @version 1.00
 *
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
/* Las rutinas de GD */
#include "gdc.h"
#include "gdchart.h"

#define MAX_LINE_SIZE 4096
#define MAX_LABEL_SIZE 128
#define MAX_COLORS 10

unsigned long colors[MAX_COLORS] =
  { 0x000000L, /* negro */
    0xF08080L, /* rojo */
    0x8DBC8FL, /* verde */
    0x0000FFL, /* azul */
    0xFFD700L, /* dorado */
    0x800080L, /* morado */
    0x800000L, /* marron */
    0x87CEEDL, /* azul cielo */
    0xFF69B4L, /* hot pink */
    0xFFA500L  /* anaranjado */
  };


void print_fs(char *infile, char *outfile, char *prog);
void print_processor(char *infile, char *outfile, char *prog);
void print_process(char *infile, char *outfile, char *prog);
void print_memory(char *infile, char *outfile, char *prog);
void print_users(char *infile, char *outfile, char *prog);
void print_network(char *infile, char *outfile, char *prog);
void print_nfs(char *infile, char *outfile, char *prog);
void print_printer(char *infile, char *outfile, char *prog);
void print_generic(char *infile, char *outfile, char *prog, char *title, char *ytitle, char* xtitle);


void print_fs(char *infile, char *outfile, char *prog) 
{
  char *title = "Sistema de Archivos";
  char *ytitle = "usado (%)";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

void print_processor(char *infile, char*outfile, char *prog)
{
  char *title = "Procesador";
  char *ytitle = "carga";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

void print_process(char *infile, char *outfile, char *prog) 
{
  char *title = "Procesos";
  char *ytitle = "No. de procesos";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

void print_memory(char *infile, char *outfile, char *prog) 
{
  char *title = "Memoria";
  char *ytitle = "usada (%)";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

void print_users(char *infile, char *outfile, char *prog) 
{
  char *title = "Usuarios";
  char *ytitle = "No. de usuarios";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

void print_network(char *infile, char *outfile, char *prog)
{
  char *title = "Red";
  char *ytitle = "paquetes (%)";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

void print_nfs(char *infile, char *outfile, char *prog) 
{
  char *title = "NFS";
  char *ytitle = "No. de sistemas";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

void print_printer(char *infile, char *outfile, char *prog) 
{
  char *title = "Impresoras";
  char *ytitle = "No. de trabajos";
  char *xtitle = "tiempo (h)";

  print_generic(infile, outfile, prog, title, ytitle, xtitle);
}

/* Esta funcin recibe el archivo de entrada con los datos a
   graficar.  Dicho archivo debe tener el siguiente formato:
   
   ----
   Num	Num
   Datos
   [Datos...]
   ----
   
   donde el primer nmero es el nmero de puntos (sobre el eje X) y el
   segundo el nmero de conjuntos a graficar (i.e. el nmero de lneas
   que aparecern en la grfica).

   Datos es una lnea completa (que debe tener el nmero de datos
   adecuados a graficar).  No hay gran chequeo de datos, por razones
   de eficiencia, as que debe ser fcil romper esta funcin.

   Recibe como segundo argumento (outfile) el nombre del archivo donde
   guardar el gif generado.  (prog sirve slo para mandar mensajes de
   error).

   Recibe tambin tres cadenas (pequeas) con los ttulos de la
   grfica, de los datos sobre el eje Y y sobre el eje X,
   respectivamente.
*/
void print_generic(char *infile, char *outfile, char *prog,
                   char *title, char *ytitle, char* xtitle) {
  FILE *outgif;
  FILE *rawdata;
  /* Estos arreglos contendrn la informacin que vamos a
   * graficar: */
  int data_points, filesystems, i, j, foo;
  float **data, **datum, *tmp;
  char **lab, **labels, **dataTemp;
  char *sep = "\t ";
  char *p, *line;
  unsigned long *sc;
        
  if((outgif = fopen( outfile, "w")) == NULL) {
    fprintf(stderr, "%s: no puedo abrir el archivo: %s\n", prog, outfile);
    exit(65);
  }
  if((rawdata = fopen( infile, "r")) == NULL) {
    fprintf(stderr, "%s: no puedo leer del archivo: %s\n", prog, infile);
  } 

  /* Ahora tenemos que leer los datos e ir guardando la
   * informacin en los arreglos adecuados.  Esta es la parte
   * truculenta y depende de que el archivo de datos tenga un
   * formato especfico muy preciso.  No hacemos chequeo de
   * errores, porque sera muy lento. */
  fscanf(rawdata, "%d\t%d\n", &data_points, &filesystems);
  
  line = (char *)malloc(MAX_LINE_SIZE);
  /* Leemos las eqtiquetas (de tiempo) del eje X */
  if ((fgets(line, MAX_LINE_SIZE, rawdata)) == NULL) {
    fprintf(stderr, "%s: Error en formato del archivo de datos (etiquetas)\n", prog);
    exit(65);
  }

  /* Partimos en tokens la cadena de las etiquetas.  El
     formato que dee tener esta lnea es cada etiqueta
     (timestamp) separada por TAB */
  labels = (char **)malloc(sizeof(char **) * data_points + 1);        
  for(lab = labels; (*lab = strsep(&line, sep)) != NULL;)
    if (**lab != '\0')
      if (++lab >= &labels[data_points])
        break;
  /* Liberamos el arreglo temporal */
  free(line);

  /* Ya conocemos el nmero de puntos y el nmero de sistemas de
   * archivos.  Le asignamos espacio a los arreglos de datos: */
  data = (float **)malloc(sizeof(float **) * filesystems + 1);
  /* Algo similar con sus colores */
  sc = (unsigned long*)malloc(sizeof(unsigned long) * filesystems + 1);
        
  for (i = 0; i < filesystems; i++){
    /* Nos comemos la lnea */
    line = (char *)malloc(MAX_LINE_SIZE);
    if((fgets(line, MAX_LINE_SIZE, rawdata)) == NULL) {
      fprintf(stderr, "%s: Error en formato del archivo de datos (datos)\n", prog);
      exit(65);
    }
          
    /* Ahora nos comemos los datos.  Primero los metemos a un
       arreglo temporal y luego dividemos la etiqueta y los
       datos en los arreglos correspondientes para el GDCHART. */
    dataTemp = (char **)malloc(sizeof(char **) * data_points + 1);
    for(lab = dataTemp; (*lab = strsep(&line, sep)) != NULL;)
      if (**lab != '\0')
        if (++lab >= &dataTemp[data_points])
          break;

    /* Ahora nos comemos los datos */
    data[i] = (float *)malloc(sizeof(float) * data_points);
    for(j = 0; j < data_points; j++) {
      foo = 0;
      sscanf(dataTemp[j], "%d", &foo);
      data[i][j] = (float)foo;
    }
    free(line); free(dataTemp);
    /* Le asignamos un color a esta lnea */
    sc[i] = colors[i % MAX_COLORS];
  }

  /* Aplanamos los datos para poder imprimirlos */
  tmp = (float*)malloc(sizeof(float) * (filesystems * data_points) + 1);
  for (i = 0; i < filesystems; i++)
    for (j = 0; j <  data_points; j++)
      tmp[i*data_points + j] = data[i][j];

  /* Lo siguiente modifica algunos parmetros de gdchart para
   * hacer "adecuada" la salida. */
  GDC_title     = title;
  GDC_ytitle    = ytitle;
  GDC_xtitle    = xtitle;
        
  GDC_BGColor   = 0xFFFFFFL;  /* background (white) */
  GDC_LineColor = 0x000000L;  /* line color (black) */
  GDC_SetColor  = &(sc[0]);   /* assign set colors */


  /* Finalmente imprimimos la grfica en el archivo de
   * salida. */
  GDC_out_graph( 500, 300,  /* short  ancho, altura */
                 outgif,    /* FILE*  open FILE pointer */
                 GDC_LINE,     /* GDC_CHART_T tipo de grfica
                                  (slo usamos GDC_LINE, o GDC_3DLINE) */
                 data_points, /* int nmero de puntos por
                                 conjunto de datos. */
                 labels, /* char*[] arreglo de etiquetas (eje X) */
                 filesystems, /* int nmero de conjuntos de datos */
                 tmp, NULL ); /* float[] conjunto de datos
                                 (aplanados a una sla dimensin) */
}


int main(int argc, char *argv[]) {

        char *client, *infile, *outfile;
        char *prog = argv[0];
        
        /* Primero los argumentos en la lnea: */
        if ( argc != 4 ) {
                fprintf(stderr, "Error: Esperaba 3 argumentos y recib: %d\nUso: %s tipo input output\n", argc, argv[0]);
                exit(65);
        } else {
                /* Qu cliente require grfica: */
                client = argv[1];
                /* archivo con los datos de entrada */
                infile = argv[2];
                /* archivo de salida (en formato GIF) */
                outfile = argv[3];
        }

        switch ( atoi(client) ) {
        case 0:
          /* File System */
          print_fs(infile, outfile, prog);
          break;
        case 1:
          /* Processor */
          print_processor(infile, outfile, prog);
          break;
        case 2:
          /* Process */
          print_process(infile, outfile, prog);
          break;
        case 3:
          /* Memoria */
          print_memory(infile, outfile, prog);
          break;
        case 4:
          /* Users */
          print_users(infile, outfile, prog);
          break;
        case 5:
          /* Network */
          print_network(infile, outfile, prog);
          break;
        case 6:
          /* NFS */
          print_nfs(infile, outfile, prog);
          break;
        case 7:
          /* Printer */
          print_printer(infile, outfile, prog);
          break;
        }

        /* Eso es todo */
        return 0;
}



/* graph.c ends here */
